#ifndef _MYSOCKET_H
#define _MYSOCKET_H

#include <tango.h>

#include <string>
#include <sstream>

#include <netdb.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/tcp.h>

namespace mysocket_ns
{

class MySocket : public virtual Tango::LogAdapter {
	public :
		MySocket(class Tango::DeviceImpl *td, const std::string host_to_connect_to,
				const unsigned short port_to_connect_to,
				const int timeout /* in msec */, bool nagle);
		~MySocket();

		bool connect(void);
		void disconnect(void);
		bool is_connected(void);
		bool check_and_reconnect(stringstream &strerr);
		void dump(const unsigned char *buff = NULL, const int len = 0);
		void flush(void);
		void interrupt(void);
		void set_timeout(unsigned int timeout_msec);
		void set_ltimeout(unsigned int ltimeout_sec); /* Linger is 1 in default configuration*/

		// ASCII oriented comunication
		bool read_until(std::stringstream& strbuffer, const char lastchar);
		int read_for(std::stringstream& strbuffer, const int numchars);
		int send_all(const std::stringstream& strbuffer);

		// Binary oriented comunication
		int send_all(const unsigned char *buff, const int numbytes);
		int read_for(unsigned char *buff, const int numbytes);

	protected:
		bool wait_data(struct timeval *tv);
		int length(void);

	private:
		class Tango::DeviceImpl *tango_device;
		std::string hostname;
		unsigned short port;
		int socketfd;
		struct timeval timeout;	// Timeout for recv()
		int pipefd[2];		// Pipe per interrupt select()
		int maxfd;		// Max file descriptor for select()
		fd_set readfds;		// Reading file descriptors
		bool interrupted;	// Flag to interrupt select()
		bool nagle;		// Status of Nagle
		int linger_timeout;	// Linger in seconds
};

} // namespace mysocket_ns

#endif // _MYSOCKET_H
